const DONOTHING = 0;
const LOCAL_TRANSFORM = 1 << 0;
const WORLD_TRANSFORM = 1 << 1;
const TRANSFORM = LOCAL_TRANSFORM | WORLD_TRANSFORM;
const UPDATE_RENDER_DATA = 1 << 2;
const OPACITY = 1 << 3;
const COLOR = 1 << 4;
const RENDER = 1 << 5;
const CUSTOM_IA_RENDER = 1 << 6;
const CHILDREN = 1 << 7;
const POST_UPDATE_RENDER_DATA = 1 << 8;
const POST_RENDER = 1 << 9;
const FINAL = 1 << 10;

var RenderFlow = [];
RenderFlow.FLAG_DONOTHING = DONOTHING;
RenderFlow.FLAG_LOCAL_TRANSFORM = LOCAL_TRANSFORM;
RenderFlow.FLAG_WORLD_TRANSFORM = WORLD_TRANSFORM;
RenderFlow.FLAG_TRANSFORM = TRANSFORM;
RenderFlow.FLAG_COLOR = COLOR;
RenderFlow.FLAG_OPACITY = OPACITY;
RenderFlow.FLAG_UPDATE_RENDER_DATA = UPDATE_RENDER_DATA;
RenderFlow.FLAG_RENDER = RENDER;
RenderFlow.FLAG_CUSTOM_IA_RENDER = CUSTOM_IA_RENDER;
RenderFlow.FLAG_CHILDREN = CHILDREN;
RenderFlow.FLAG_POST_UPDATE_RENDER_DATA = POST_UPDATE_RENDER_DATA;
RenderFlow.FLAG_POST_RENDER = POST_RENDER;
RenderFlow.FLAG_FINAL = FINAL;

const StencilMaterial = cc.renderer.renderEngine.StencilMaterial;
// const Graphics = require('../graphics/graphics');


let myMask = cc.Class({
    // name: 'cc.myMask',
    extends: cc.RenderComponent,

    editor: CC_EDITOR && {
        menu: 'i18n:MAIN_MENU.component.renderers/myMask',
        help: 'i18n:COMPONENT.help_url.mymask',
        inspector: 'packages://inspector/inspectors/comps/mymask.js'
    },

    ctor () {
        this._graphics = null;
        this._clearGraphics = null;
        this._assembler = cc.Mask._assembler ;
        this._postAssembler = cc.Mask._postAssembler ;
    },

    properties: {

        inverted: {
            default: false,
            type: cc.Boolean,
            tooltip: CC_DEV && 'i18n:COMPONENT.mask.inverted',
            notify: function () {
                if (cc.game.renderType === cc.game.RENDER_TYPE_CANVAS) {
                    cc.warnID(4202);
                    return;
                }
            }
        }
    },



    onLoad () {
        this._createGraphics();
    },

    onRestore () {
        this._createGraphics();
        this._updateGraphics();

    },

    onEnable () {
        this._super();
        this._updateGraphics();


        this.node.on(cc.Node.EventType.POSITION_CHANGED, this._updateGraphics, this);
        this.node.on(cc.Node.EventType.ROTATION_CHANGED, this._updateGraphics, this);
        this.node.on(cc.Node.EventType.SCALE_CHANGED, this._updateGraphics, this);
        this.node.on(cc.Node.EventType.SIZE_CHANGED, this._updateGraphics, this);
        this.node.on(cc.Node.EventType.ANCHOR_CHANGED, this._updateGraphics, this);

        this.node._renderFlag |= RenderFlow.FLAG_POST_RENDER;
        this._activateMaterial();
    },

    onDisable () {
        this._super();

        this.node.off(cc.Node.EventType.POSITION_CHANGED, this._updateGraphics, this);
        this.node.off(cc.Node.EventType.ROTATION_CHANGED, this._updateGraphics, this);
        this.node.off(cc.Node.EventType.SCALE_CHANGED, this._updateGraphics, this);
        this.node.off(cc.Node.EventType.SIZE_CHANGED, this._updateGraphics, this);
        this.node.off(cc.Node.EventType.ANCHOR_CHANGED, this._updateGraphics, this);

        this.node._renderFlag &= ~RenderFlow.FLAG_POST_RENDER;
    },

    onDestroy () {
        this._super();
        this._removeGraphics();
    },

    _resizeNodeToTargetNode: CC_EDITOR && function () {
        if(this.spriteFrame) {
            let rect = this.spriteFrame.getRect();
            this.node.setContentSize(rect.width, rect.height);
        }
    },

    _onTextureLoaded () {
        // Mark render data dirty
        if (this._renderData) {
            this._renderData.uvDirty = true;
            this._renderData.vertDirty = true;
            this.markForUpdateRenderData(true);
        }
        // Reactivate material
        if (this.enabledInHierarchy) {
            this._activateMaterial();
        }
    },

    _applySpriteFrame (oldFrame) {
        
    },

    _activateMaterial () {


        // WebGL
        if (cc.game.renderType !== cc.game.RENDER_TYPE_CANVAS) {
            // Init material
            if (!this._material) {
                this._material = new StencilMaterial();
            }

            // Reset material
            this._material.useModel = true;
            this._material.useTexture = false;
            this._material.useColor = false;

        }

        this.markForRender(true);
    },

    _createGraphics () {
        if (!this._graphics) {
            this._graphics = new cc.Graphics();
            this._graphics.node = this.node;
            this._graphics.lineWidth = 0;
            this._graphics.strokeColor = cc.color(0, 0, 0, 0);
        }

        if (!this._clearGraphics) {
            this._clearGraphics = new cc.Graphics();
            this._clearGraphics.node = new cc.Node();
            this._clearGraphics._activateMaterial();
            this._clearGraphics.lineWidth = 0;
            this._clearGraphics.rect(0, 0, cc.visibleRect.width, cc.visibleRect.height);
            this._clearGraphics.fill();
        }
    },

    _setMyData(point){
        this.l_point = point
    },

    _updateGraphics () {
        let node = this.node;
        let graphics = this._graphics;

        if(graphics){

        }
        else{
            this._createGraphics()
            graphics = this._graphics;
        }
        // Share render data with graphics content
        graphics.clear(false);
        let width = node._contentSize.width;
        let height = node._contentSize.height;
        let x = -width * node._anchorPoint.x;
        let y = -height * node._anchorPoint.y;

        if(this.l_point){

        }
        else{
            var point = []
            point.push(cc.v2(0,0))
            point.push(cc.v2(200, 200))
            point.push(cc.v2(200, 0))
            this.l_point = point
        }
        for (var i = 0; i < this.l_point.length; i++) {
            var point = this.l_point[i]
            if (i === 0) {
                graphics.moveTo(point.x, point.y);
            }
            else {
                graphics.lineTo(point.x, point.y);
            }
        }



        graphics.close();


        // if (this._type === MaskType.RECT) {
        //     graphics.rect(x, y, width, height);
        // }
        // else if (this._type === MaskType.ELLIPSE) {
        //     let center = cc.v2(x + width / 2, y + height / 2);
        //     let radius = {
        //         x: width / 2,
        //         y: height / 2
        //     };
        //     let points = _calculateCircle(center, radius, this._segments);
        //     for (let i = 0; i < points.length; ++i) {
        //         let point = points[i];
        //         if (i === 0) {
        //             graphics.moveTo(point.x, point.y);
        //         }
        //         else {
        //             graphics.lineTo(point.x, point.y);
        //         }
        //     }
        //     graphics.close();
        // }
        if (cc.game.renderType === cc.game.RENDER_TYPE_CANVAS) {
            graphics.stroke();
        }
        else {
            graphics.fill();
        }
    },

    _removeGraphics () {
        if (this._graphics) {
            this._graphics.destroy();
        }

        if (this._clearGraphics) {
            this._clearGraphics.destroy();
        }
    },

    _hitTest (cameraPt) {
        // let node = this.node;
        // let size = node.getContentSize(),
        //     w = size.width,
        //     h = size.height,
        //     testPt = _vec2_temp;
        
        // node._updateWorldMatrix();
        // math.mat4.invert(_mat4_temp, node._worldMatrix);
        // math.vec2.transformMat4(testPt, cameraPt, _mat4_temp);
        // testPt.x += node._anchorPoint.x * w;
        // testPt.y += node._anchorPoint.y * h;


        // else if (this.type === MaskType.ELLIPSE) {
        //     let rx = w / 2, ry = h / 2;
        //     let px = testPt.x - 0.5 * w, py = testPt.y - 0.5 * h;
        //     return px * px / (rx * rx) + py * py / (ry * ry) < 1;
        // }
    },

    markForUpdateRenderData (enable) {
        if (enable && this.enabledInHierarchy) {
            this.node._renderFlag |= RenderFlow.FLAG_UPDATE_RENDER_DATA;
        }
        else if (!enable) {
            this.node._renderFlag &= ~RenderFlow.FLAG_UPDATE_RENDER_DATA;
        }
    },

    markForRender (enable) {
        if (enable && this.enabledInHierarchy) {
            this.node._renderFlag |= (RenderFlow.FLAG_RENDER | RenderFlow.FLAG_UPDATE_RENDER_DATA | 
                                      RenderFlow.FLAG_POST_RENDER);
        }
        else if (!enable) {
            this.node._renderFlag &= ~(RenderFlow.FLAG_RENDER | RenderFlow.FLAG_POST_RENDER);
        }
    },

    disableRender () {
        this.node._renderFlag &= ~(RenderFlow.FLAG_RENDER | RenderFlow.FLAG_UPDATE_RENDER_DATA | 
                                   RenderFlow.FLAG_POST_RENDER);
    },

    _calDynamicAtlas ()
    {

    }
});



cc.myMask = module.exports = myMask;
